@@ -1,24 +1,7 @@
 #
 # Copyright © 2021 Google, Inc.
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice (including the next
-# paragraph) shall be included in all copies or substantial portions of the
-# Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
-# IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 
 from mako.template import Template
 import sys
@@ -79,20 +62,20 @@ def add_gpus(ids, info):
 
 class GPUId(object):
     def __init__(self, gpu_id = None, chip_id = None, name=None):
-        if chip_id == None:
-            assert(gpu_id != None)
+        if chip_id is None:
+            assert(gpu_id is not None)
             val = gpu_id
             core = int(val / 100)
-            val -= (core * 100);
-            major = int(val / 10);
+            val -= (core * 100)
+            major = int(val / 10)
             val -= (major * 10)
             minor = val
             chip_id = (core << 24) | (major << 16) | (minor << 8) | 0xff
         self.chip_id = chip_id
-        if gpu_id == None:
+        if gpu_id is None:
             gpu_id = 0
         self.gpu_id = gpu_id
-        if name == None:
+        if name is None:
             assert(gpu_id != 0)
             name = "FD%d" % gpu_id
         self.name = name
@@ -119,6 +102,7 @@ class GPUInfo(Struct):
                  tile_align_w, tile_align_h,
                  tile_max_w, tile_max_h, num_vsc_pipes,
                  cs_shared_mem_size, num_sp_cores, wave_granularity, fibers_per_sp,
+                 highest_bank_bit = 0, ubwc_swizzle = 0x7, macrotile_mode = 0,
                  threadsize_base = 64, max_waves = 16):
         self.chip          = chip.value
         self.gmem_align_w  = gmem_align_w
@@ -134,6 +118,9 @@ class GPUInfo(Struct):
         self.fibers_per_sp = fibers_per_sp
         self.threadsize_base = threadsize_base
         self.max_waves     = max_waves
+        self.highest_bank_bit = highest_bank_bit
+        self.ubwc_swizzle = ubwc_swizzle
+        self.macrotile_mode = macrotile_mode
 
         s.gpu_infos.append(self)
 
@@ -146,8 +133,9 @@ class A6xxGPUInfo(GPUInfo):
     def __init__(self, chip, template, num_ccu,
                  tile_align_w, tile_align_h, num_vsc_pipes,
                  cs_shared_mem_size, wave_granularity, fibers_per_sp,
-                 magic_regs, raw_magic_regs = None, threadsize_base = 64,
-                 max_waves = 16):
+                 magic_regs, raw_magic_regs = None, highest_bank_bit = 15,
+                 ubwc_swizzle = 0x6, macrotile_mode = 1,
+                 threadsize_base = 64, max_waves = 16):
         if chip == CHIP.A6XX:
             tile_max_w   = 1024 # max_bitfield_val(5, 0, 5)
             tile_max_h   = max_bitfield_val(14, 8, 4) # 1008
@@ -165,6 +153,9 @@ class A6xxGPUInfo(GPUInfo):
                          num_sp_cores = num_ccu, # The # of SP cores seems to always match # of CCU
                          wave_granularity   = wave_granularity,
                          fibers_per_sp      = fibers_per_sp,
+                         highest_bank_bit = highest_bank_bit,
+                         ubwc_swizzle = ubwc_swizzle,
+                         macrotile_mode = macrotile_mode,
                          threadsize_base    = threadsize_base,
                          max_waves    = max_waves)
 
@@ -181,7 +172,7 @@ class A6xxGPUInfo(GPUInfo):
         if raw_magic_regs:
             self.a6xx.magic_raw = [[int(r[0]), r[1]] for r in raw_magic_regs]
 
-        templates = template if type(template) is list else [template]
+        templates = template if isinstance(template, list) else [template]
         for template in templates:
             template.apply_props(self)
 
@@ -217,6 +208,7 @@ add_gpus([
         GPUId(320),
         GPUId(330),
         GPUId(chip_id=0x03000512, name="FD305B"),
+        GPUId(chip_id=0x03000620, name="FD306A"),
     ], GPUInfo(
         CHIP.A3XX,
         gmem_align_w = 32,  gmem_align_h = 32,
@@ -250,6 +242,7 @@ add_gpus([
     ))
 
 add_gpus([
+        GPUId(505),
         GPUId(506),
         GPUId(508),
         GPUId(509),
@@ -264,6 +257,7 @@ add_gpus([
         num_sp_cores = 1,
         wave_granularity = 2,
         fibers_per_sp = 64 * 16, # Lowest number that didn't fault on spillall fs-varying-array-mat4-col-row-rd.
+        highest_bank_bit = 14,
         threadsize_base = 32,
     ))
 
@@ -281,6 +275,7 @@ add_gpus([
         num_sp_cores = 2,
         wave_granularity = 2,
         fibers_per_sp = 64 * 16, # Lowest number that didn't fault on spillall fs-varying-array-mat4-col-row-rd.
+        highest_bank_bit = 14,
         threadsize_base = 32,
     ))
 
@@ -298,6 +293,7 @@ add_gpus([
         num_sp_cores = 4,
         wave_granularity = 2,
         fibers_per_sp = 64 * 16, # Lowest number that didn't fault on spillall fs-varying-array-mat4-col-row-rd.
+        highest_bank_bit = 15,
         threadsize_base = 32,
     ))
 
@@ -345,9 +341,9 @@ a6xx_base = A6XXProps(
     )
 
 
-# a6xx can be divided into distinct sub-generations, where certain device-
-# info parameters are keyed to the sub-generation.  These templates reduce
-# the copypaste
+# a6xx and a7xx can be divided into distinct sub-generations, where certain
+# device-info parameters are keyed to the sub-generation.  These templates
+# reduce the copypaste
 
 a6xx_gen1_low = A6XXProps(
         reg_size_vec4 = 48,
@@ -404,6 +400,7 @@ a6xx_gen3 = A6XXProps(
         has_per_view_viewport = True,
         has_scalar_alu = True,
         has_early_preamble = True,
+        prede_nop_quirk = True,
     )
 
 a6xx_gen4 = A6XXProps(
@@ -420,7 +417,7 @@ a6xx_gen4 = A6XXProps(
         has_cp_reg_write = False,
         has_8bpp_ubwc = False,
         has_lpac = True,
-        has_shading_rate = True,
+        has_legacy_pipeline_shading_rate = True,
         has_getfiberid = True,
         has_dp2acc = True,
         has_dp4acc = True,
@@ -431,13 +428,13 @@ a6xx_gen4 = A6XXProps(
         has_scalar_alu = True,
         has_isam_v = True,
         has_ssbo_imm_offsets = True,
+        has_ubwc_linear_mipmap_fallback = True,
         # TODO: there seems to be a quirk where at least rcp can't be in an
         # early preamble. a660 at least is affected.
         #has_early_preamble = True,
-    )
-
-a6xx_a690_quirk = A6XXProps(
-        broken_ds_ubwc_quirk = True,
+        prede_nop_quirk = True,
+        predtf_nop_quirk = True,
+        has_sad = True,
     )
 
 add_gpus([
@@ -455,6 +452,9 @@ add_gpus([
         cs_shared_mem_size = 16 * 1024,
         wave_granularity = 1,
         fibers_per_sp = 128 * 16,
+        highest_bank_bit = 13,
+        ubwc_swizzle = 0x7,
+        macrotile_mode = 0,
         magic_regs = dict(
             PC_POWER_CNTL = 0,
             TPL1_DBG_ECO_CNTL = 0,
@@ -487,6 +487,8 @@ add_gpus([
         cs_shared_mem_size = 32 * 1024,
         wave_granularity = 2,
         fibers_per_sp = 128 * 16,
+        highest_bank_bit = 14,
+        macrotile_mode = 0,
         magic_regs = dict(
             PC_POWER_CNTL = 0,
             TPL1_DBG_ECO_CNTL = 0x00108000,
@@ -533,6 +535,36 @@ add_gpus([
         )
     ))
 
+add_gpus([
+        GPUId(chip_id=0xffff06020100, name="FD621"),
+    ], A6xxGPUInfo(
+        CHIP.A6XX,
+        [a6xx_base, a6xx_gen3, A6XXProps(lrz_track_quirk = False)],
+        num_ccu = 2,
+        tile_align_w = 96,
+        tile_align_h = 16,
+        num_vsc_pipes = 32,
+        cs_shared_mem_size = 32 * 1024,
+        wave_granularity = 2,
+        fibers_per_sp = 128 * 2 * 16,
+        magic_regs = dict(
+            PC_POWER_CNTL = 0,
+            # this seems to be a chicken bit that fixes cubic filtering:
+            TPL1_DBG_ECO_CNTL = 0x01008000,
+            GRAS_DBG_ECO_CNTL = 0x0,
+            SP_CHICKEN_BITS = 0x00001400,
+            # UCHE_CLIENT_PF = 0x00000004,
+            PC_MODE_CNTL = 0x1f,
+            SP_DBG_ECO_CNTL = 0x03000000,
+            RB_DBG_ECO_CNTL = 0x04100000,
+            RB_DBG_ECO_CNTL_blit = 0x04100000,
+            HLSQ_DBG_ECO_CNTL = 0x0,
+            RB_UNKNOWN_8E01 = 0x0,
+            VPC_DBG_ECO_CNTL = 0x02000000,
+            UCHE_UNKNOWN_0E12 = 0x00000001
+        )
+    ))
+
 add_gpus([
         GPUId(630),
     ], A6xxGPUInfo(
@@ -545,6 +577,8 @@ add_gpus([
         cs_shared_mem_size = 32 * 1024,
         wave_granularity = 2,
         fibers_per_sp = 128 * 16,
+        highest_bank_bit = 15,
+        macrotile_mode = 0,
         magic_regs = dict(
             PC_POWER_CNTL = 1,
             TPL1_DBG_ECO_CNTL = 0x00108000,
@@ -574,6 +608,8 @@ add_gpus([
         cs_shared_mem_size = 32 * 1024,
         wave_granularity = 2,
         fibers_per_sp = 128 * 4 * 16,
+        highest_bank_bit = 15,
+        macrotile_mode = 0,
         magic_regs = dict(
             PC_POWER_CNTL = 1,
             TPL1_DBG_ECO_CNTL = 0x00008000,
@@ -603,6 +639,8 @@ add_gpus([
         cs_shared_mem_size = 32 * 1024,
         wave_granularity = 2,
         fibers_per_sp = 128 * 4 * 16,
+        highest_bank_bit = 15,
+        macrotile_mode = 0,
         magic_regs = dict(
             PC_POWER_CNTL = 3,
             TPL1_DBG_ECO_CNTL = 0x00108000,
@@ -632,6 +670,7 @@ add_gpus([
         cs_shared_mem_size = 32 * 1024,
         wave_granularity = 2,
         fibers_per_sp = 128 * 2 * 16,
+        highest_bank_bit = 16,
         magic_regs = dict(
             PC_POWER_CNTL = 2,
             # this seems to be a chicken bit that fixes cubic filtering:
@@ -651,6 +690,7 @@ add_gpus([
     ))
 
 add_gpus([
+        # These are all speedbins/variants of A635
         GPUId(chip_id=0x00be06030500, name="Adreno 8c Gen 3"),
         GPUId(chip_id=0x007506030500, name="Adreno 7c+ Gen 3"),
         GPUId(chip_id=0x006006030500, name="Adreno 7c+ Gen 3 Lite"),
@@ -667,6 +707,7 @@ add_gpus([
         cs_shared_mem_size = 32 * 1024,
         wave_granularity = 2,
         fibers_per_sp = 128 * 2 * 16,
+        highest_bank_bit = 14,
         magic_regs = dict(
             PC_POWER_CNTL = 1,
             TPL1_DBG_ECO_CNTL = 0x05008000,
@@ -696,6 +737,7 @@ add_gpus([
         cs_shared_mem_size = 32 * 1024,
         wave_granularity = 2,
         fibers_per_sp = 128 * 2 * 16,
+        highest_bank_bit = 16,
         magic_regs = dict(
             PC_POWER_CNTL = 2,
             TPL1_DBG_ECO_CNTL = 0x05008000,
@@ -714,7 +756,8 @@ add_gpus([
     ))
 
 add_gpus([
-        GPUId(chip_id=0x6060201, name="FD644"),
+        GPUId(chip_id=0x6060201, name="FD644"), # Called A662 in kgsl
+        GPUId(chip_id=0xffff06060300, name="FD663"),
     ], A6xxGPUInfo(
         CHIP.A6XX,
         [a6xx_base, a6xx_gen4],
@@ -747,7 +790,7 @@ add_gpus([
         GPUId(chip_id=0xffff06090000, name="FD690"), # Default no-speedbin fallback
     ], A6xxGPUInfo(
         CHIP.A6XX,
-        [a6xx_base, a6xx_gen4, a6xx_a690_quirk],
+        [a6xx_base, a6xx_gen4, A6XXProps(broken_ds_ubwc_quirk = True)],
         num_ccu = 8,
         tile_align_w = 64,
         tile_align_h = 32,
@@ -755,6 +798,7 @@ add_gpus([
         cs_shared_mem_size = 32 * 1024,
         wave_granularity = 2,
         fibers_per_sp = 128 * 2 * 16,
+        highest_bank_bit = 16,
         magic_regs = dict(
             PC_POWER_CNTL = 7,
             TPL1_DBG_ECO_CNTL = 0x04c00000,
@@ -803,7 +847,6 @@ a7xx_base = A6XXProps(
         has_separate_chroma_filter = True,
         has_sample_locations = True,
         has_lpac = True,
-        has_shading_rate = True,
         has_getfiberid = True,
         has_dp2acc = True,
         has_dp4acc = True,
@@ -818,22 +861,21 @@ a7xx_base = A6XXProps(
         has_isam_v = True,
         has_ssbo_imm_offsets = True,
         has_early_preamble = True,
+        has_attachment_shading_rate = True,
+        has_ubwc_linear_mipmap_fallback = True,
+        prede_nop_quirk = True,
+        predtf_nop_quirk = True,
+        has_sad = True,
     )
 
-a7xx_725 = A7XXProps(
-        cmdbuf_start_a725_quirk = True,
+a7xx_gen1 = A7XXProps(
         supports_ibo_ubwc = True,
         fs_must_have_non_zero_constlen_quirk = True,
         enable_tp_ubwc_flag_hint = True,
+        reading_shading_rate_requires_smask_quirk = True,
     )
 
-a7xx_730 = A7XXProps(
-        supports_ibo_ubwc = True,
-        fs_must_have_non_zero_constlen_quirk = True,
-        enable_tp_ubwc_flag_hint = True,
-    )
-
-a7xx_740 = A7XXProps(
+a7xx_gen2 = A7XXProps(
         stsc_duplication_quirk = True,
         has_event_write_sample_count = True,
         ubwc_unorm_snorm_int_compatible = True,
@@ -842,19 +884,13 @@ a7xx_740 = A7XXProps(
         # Most devices with a740 have blob v6xx which doesn't have
         # this hint set. Match them for better compatibility by default.
         enable_tp_ubwc_flag_hint = False,
+        has_64b_ssbo_atomics = True,
+        has_primitive_shading_rate = True,
+        reading_shading_rate_requires_smask_quirk = True,
+        has_ray_intersection = True,
     )
 
-a7xx_740_a32 = A7XXProps(
-        cmdbuf_start_a725_quirk = True,
-        stsc_duplication_quirk = True,
-        has_event_write_sample_count = True,
-        ubwc_unorm_snorm_int_compatible = True,
-        supports_ibo_ubwc = True,
-        fs_must_have_non_zero_constlen_quirk = True,
-        enable_tp_ubwc_flag_hint = False,
-    )
-
-a7xx_750 = A7XXProps(
+a7xx_gen3 = A7XXProps(
         has_event_write_sample_count = True,
         load_inline_uniforms_via_preamble_ldgk = True,
         load_shader_consts_via_preamble = True,
@@ -862,14 +898,21 @@ a7xx_750 = A7XXProps(
         sysmem_vpc_attr_buf_size = 0x20000,
         gmem_vpc_attr_buf_size = 0xc000,
         ubwc_unorm_snorm_int_compatible = True,
-        # a750 has a bug where writing and then reading a UBWC-compressed IBO
-        # requires flushing UCHE. This is reproducible in many CTS tests, for
-        # example dEQP-VK.image.load_store.with_format.2d.*. Disable this for
-        # now.
-        #supports_ibo_ubwc = True,
-        no_gs_hw_binning_quirk = True,
+        supports_ibo_ubwc = True,
+        has_generic_clear = True,
+        r8g8_faulty_fast_clear_quirk = True,
         gs_vpc_adjacency_quirk = True,
         storage_8bit = True,
+        ubwc_all_formats_compatible = True,
+        has_compliant_dp4acc = True,
+        ubwc_coherency_quirk = True,
+        has_persistent_counter = True,
+        has_64b_ssbo_atomics = True,
+        has_primitive_shading_rate = True,
+        has_ray_intersection = True,
+        has_sw_fuse = True,
+        has_rt_workaround = True,
+        has_alias_rt=True,
     )
 
 a730_magic_regs = dict(
@@ -922,15 +965,76 @@ a730_raw_magic_regs = [
 
         [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_80A7, 0x00000000],
 
-        [A6XXRegs.REG_A7XX_HLSQ_UNKNOWN_A9AC, 0x00000000],
         [A6XXRegs.REG_A7XX_RB_UNKNOWN_8E79,   0x00000000],
         [A6XXRegs.REG_A7XX_RB_UNKNOWN_8899,   0x00000000],
         [A6XXRegs.REG_A7XX_RB_UNKNOWN_88F5,   0x00000000],
+    ]
 
-        # Shading rate group
-        [A6XXRegs.REG_A6XX_RB_UNKNOWN_88F4,   0x00000000],
-        [A6XXRegs.REG_A7XX_HLSQ_UNKNOWN_A9AD, 0x00000000],
-        [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_80F4, 0x00000000],
+a740_magic_regs = dict(
+        # PC_POWER_CNTL = 7,
+        TPL1_DBG_ECO_CNTL = 0x11100000,
+        GRAS_DBG_ECO_CNTL = 0x00004800,
+        SP_CHICKEN_BITS = 0x10001400,
+        UCHE_CLIENT_PF = 0x00000084,
+        # Blob uses 0x1f or 0x1f1f, however these values cause vertices
+        # corruption in some tests.
+        PC_MODE_CNTL = 0x0000003f,
+        SP_DBG_ECO_CNTL = 0x10000000,
+        RB_DBG_ECO_CNTL = 0x00000000,
+        RB_DBG_ECO_CNTL_blit = 0x00000000,  # is it even needed?
+        # HLSQ_DBG_ECO_CNTL = 0x0,
+        RB_UNKNOWN_8E01 = 0x0,
+        VPC_DBG_ECO_CNTL = 0x02000000,
+        UCHE_UNKNOWN_0E12 = 0x00000000,
+
+        RB_UNKNOWN_8E06 = 0x02080000,
+    )
+
+a740_raw_magic_regs = [
+        [A6XXRegs.REG_A6XX_UCHE_CACHE_WAYS, 0x00040004],
+        [A6XXRegs.REG_A6XX_TPL1_DBG_ECO_CNTL1, 0x00040724],
+
+        [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE08, 0x00000400],
+        [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE09, 0x00430800],
+        [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE0A, 0x00000000],
+        [A6XXRegs.REG_A7XX_UCHE_UNKNOWN_0E10, 0x00000000],
+        [A6XXRegs.REG_A7XX_UCHE_UNKNOWN_0E11, 0x00000000],
+        [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE6C, 0x00000000],
+        [A6XXRegs.REG_A6XX_PC_DBG_ECO_CNTL, 0x00100000],
+        [A6XXRegs.REG_A7XX_PC_UNKNOWN_9E24, 0x21585600],
+        [A6XXRegs.REG_A7XX_VFD_UNKNOWN_A600, 0x00008000],
+        [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE06, 0x00000000],
+        [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE6A, 0x00000000],
+        [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE6B, 0x00000080],
+        [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE73, 0x00000000],
+        [A6XXRegs.REG_A7XX_SP_UNKNOWN_AB02, 0x00000000],
+        [A6XXRegs.REG_A7XX_SP_UNKNOWN_AB01, 0x00000000],
+        [A6XXRegs.REG_A7XX_SP_UNKNOWN_AB22, 0x00000000],
+        [A6XXRegs.REG_A7XX_SP_UNKNOWN_B310, 0x00000000],
+
+        [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8120, 0x09510840],
+        [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8121, 0x00000a62],
+
+        [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8009, 0x00000000],
+        [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_800A, 0x00000000],
+        [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_800B, 0x00000000],
+        [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_800C, 0x00000000],
+
+        [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE2,   0x00000000],
+        [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE2+1, 0x00000000],
+        [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE4,   0x00000000],
+        [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE4+1, 0x00000000],
+        [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE6,   0x00000000],
+        [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE6+1, 0x00000000],
+
+        [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_80A7, 0x00000000],
+
+        [A6XXRegs.REG_A7XX_RB_UNKNOWN_8E79,   0x00000000],
+        [A6XXRegs.REG_A7XX_RB_UNKNOWN_8899,   0x00000000],
+        [A6XXRegs.REG_A7XX_RB_UNKNOWN_88F5,   0x00000000],
+        [A6XXRegs.REG_A7XX_RB_UNKNOWN_8C34,   0x00000000],
+
+        [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8008, 0x00000000],
     ]
 
 add_gpus([
@@ -939,7 +1043,7 @@ add_gpus([
         GPUId(chip_id=0xffff07030002, name="FD725"),
     ], A6xxGPUInfo(
         CHIP.A7XX,
-        [a7xx_base, a7xx_725],
+        [a7xx_base, a7xx_gen1, A7XXProps(cmdbuf_start_a725_quirk = True)],
         num_ccu = 4,
         tile_align_w = 64,
         tile_align_h = 32,
@@ -947,6 +1051,7 @@ add_gpus([
         cs_shared_mem_size = 32 * 1024,
         wave_granularity = 2,
         fibers_per_sp = 128 * 2 * 16,
+        highest_bank_bit = 16,
         magic_regs = a730_magic_regs,
         raw_magic_regs = a730_raw_magic_regs,
     ))
@@ -956,7 +1061,7 @@ add_gpus([
         GPUId(chip_id=0xffff07030001, name="FD730"), # Default no-speedbin fallback
     ], A6xxGPUInfo(
         CHIP.A7XX,
-        [a7xx_base, a7xx_730],
+        [a7xx_base, a7xx_gen1],
         num_ccu = 4,
         tile_align_w = 64,
         tile_align_h = 32,
@@ -964,21 +1069,17 @@ add_gpus([
         cs_shared_mem_size = 32 * 1024,
         wave_granularity = 2,
         fibers_per_sp = 128 * 2 * 16,
+        highest_bank_bit = 16,
         magic_regs = a730_magic_regs,
         raw_magic_regs = a730_raw_magic_regs,
     ))
 
 add_gpus([
-        GPUId(740), # Deprecated, used for dev kernels.
-        GPUId(chip_id=0x43050a01, name="FD740"), # KGSL, no speedbin data
-        GPUId(chip_id=0xffff43050a01, name="FD740"), # Default no-speedbin fallback
-        GPUId(chip_id=0x43050B00, name="FD740"), # Quest 3
-        GPUId(chip_id=0xffff43050B00, name="FD740"),
-        GPUId(chip_id=0xffff43050c01, name="Adreno X1-85"),
+        GPUId(chip_id=0x43030B00, name="FD735")
     ], A6xxGPUInfo(
         CHIP.A7XX,
-        [a7xx_base, a7xx_740],
-        num_ccu = 6,
+        [a7xx_base, a7xx_gen2, A7XXProps(enable_tp_ubwc_flag_hint = True)],
+        num_ccu = 3,
         tile_align_w = 96,
         tile_align_h = 32,
         num_vsc_pipes = 32,
@@ -986,18 +1087,14 @@ add_gpus([
         wave_granularity = 2,
         fibers_per_sp = 128 * 2 * 16,
         magic_regs = dict(
-            # PC_POWER_CNTL = 7,
             TPL1_DBG_ECO_CNTL = 0x11100000,
             GRAS_DBG_ECO_CNTL = 0x00004800,
             SP_CHICKEN_BITS = 0x10001400,
             UCHE_CLIENT_PF = 0x00000084,
-            # Blob uses 0x1f or 0x1f1f, however these values cause vertices
-            # corruption in some tests.
-            PC_MODE_CNTL = 0x0000003f,
+            PC_MODE_CNTL = 0x0000001f,
             SP_DBG_ECO_CNTL = 0x10000000,
-            RB_DBG_ECO_CNTL = 0x00000000,
-            RB_DBG_ECO_CNTL_blit = 0x00000000,  # is it even needed?
-            # HLSQ_DBG_ECO_CNTL = 0x0,
+            RB_DBG_ECO_CNTL = 0x00000001,
+            RB_DBG_ECO_CNTL_blit = 0x00000001,  # is it even needed?
             RB_UNKNOWN_8E01 = 0x0,
             VPC_DBG_ECO_CNTL = 0x02000000,
             UCHE_UNKNOWN_0E12 = 0x00000000,
@@ -1005,7 +1102,7 @@ add_gpus([
             RB_UNKNOWN_8E06 = 0x02080000,
         ),
         raw_magic_regs = [
-            [A6XXRegs.REG_A6XX_UCHE_CACHE_WAYS, 0x00040004],
+            [A6XXRegs.REG_A6XX_UCHE_CACHE_WAYS, 0x00000000],
             [A6XXRegs.REG_A6XX_TPL1_DBG_ECO_CNTL1, 0x00040724],
 
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE08, 0x00000400],
@@ -1015,7 +1112,7 @@ add_gpus([
             [A6XXRegs.REG_A7XX_UCHE_UNKNOWN_0E11, 0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE6C, 0x00000000],
             [A6XXRegs.REG_A6XX_PC_DBG_ECO_CNTL, 0x00100000],
-            [A6XXRegs.REG_A7XX_PC_UNKNOWN_9E24, 0x21585600],
+            [A6XXRegs.REG_A7XX_PC_UNKNOWN_9E24, 0x01585600],
             [A6XXRegs.REG_A7XX_VFD_UNKNOWN_A600, 0x00008000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE06, 0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE6A, 0x00000000],
@@ -1043,28 +1140,58 @@ add_gpus([
 
             [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_80A7, 0x00000000],
 
-            [A6XXRegs.REG_A7XX_HLSQ_UNKNOWN_A9AC, 0x00000000],
             [A6XXRegs.REG_A7XX_RB_UNKNOWN_8E79,   0x00000000],
             [A6XXRegs.REG_A7XX_RB_UNKNOWN_8899,   0x00000000],
             [A6XXRegs.REG_A7XX_RB_UNKNOWN_88F5,   0x00000000],
             [A6XXRegs.REG_A7XX_RB_UNKNOWN_8C34,   0x00000000],
 
-            # Shading rate group
-            [A6XXRegs.REG_A6XX_RB_UNKNOWN_88F4,   0x00000000],
-            [A6XXRegs.REG_A7XX_HLSQ_UNKNOWN_A9AD, 0x00000000],
             [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8008, 0x00000000],
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_80F4, 0x00000000],
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_80F5, 0x00000000],
         ],
     ))
 
+add_gpus([
+        GPUId(740), # Deprecated, used for dev kernels.
+        GPUId(chip_id=0x43050a01, name="FD740"), # KGSL, no speedbin data
+        GPUId(chip_id=0xffff43050a01, name="FD740"), # Default no-speedbin fallback
+    ], A6xxGPUInfo(
+        CHIP.A7XX,
+        [a7xx_base, a7xx_gen2],
+        num_ccu = 6,
+        tile_align_w = 96,
+        tile_align_h = 32,
+        num_vsc_pipes = 32,
+        cs_shared_mem_size = 32 * 1024,
+        wave_granularity = 2,
+        fibers_per_sp = 128 * 2 * 16,
+        highest_bank_bit = 16,
+        magic_regs = a740_magic_regs,
+        raw_magic_regs = a740_raw_magic_regs,
+    ))
+
+add_gpus([
+        GPUId(chip_id=0xffff43050c01, name="Adreno X1-85"),
+    ], A6xxGPUInfo(
+        CHIP.A7XX,
+        [a7xx_base, a7xx_gen2, A7XXProps(compute_constlen_quirk = True)],
+        num_ccu = 6,
+        tile_align_w = 96,
+        tile_align_h = 32,
+        num_vsc_pipes = 32,
+        cs_shared_mem_size = 32 * 1024,
+        wave_granularity = 2,
+        fibers_per_sp = 128 * 2 * 16,
+        highest_bank_bit = 16,
+        magic_regs = a740_magic_regs,
+        raw_magic_regs = a740_raw_magic_regs,
+    ))
+
 # Values from blob v676.0
 add_gpus([
         GPUId(chip_id=0x43050a00, name="FDA32"), # Adreno A32 (G3x Gen 2)
         GPUId(chip_id=0xffff43050a00, name="FDA32"),
     ], A6xxGPUInfo(
         CHIP.A7XX,
-        [a7xx_base, a7xx_740_a32],
+        [a7xx_base, a7xx_gen2, A7XXProps(cmdbuf_start_a725_quirk = True)],
         num_ccu = 6,
         tile_align_w = 96,
         tile_align_h = 32,
@@ -1072,25 +1199,7 @@ add_gpus([
         cs_shared_mem_size = 32 * 1024,
         wave_granularity = 2,
         fibers_per_sp = 128 * 2 * 16,
-        magic_regs = dict(
-            # PC_POWER_CNTL = 7,
-            TPL1_DBG_ECO_CNTL = 0x11100000,
-            GRAS_DBG_ECO_CNTL = 0x00004800,
-            SP_CHICKEN_BITS = 0x10001400,
-            UCHE_CLIENT_PF = 0x00000084,
-            # Blob uses 0x1f or 0x1f1f, however these values cause vertices
-            # corruption in some tests.
-            PC_MODE_CNTL = 0x0000003f,
-            SP_DBG_ECO_CNTL = 0x10000000,
-            RB_DBG_ECO_CNTL = 0x00000000,
-            RB_DBG_ECO_CNTL_blit = 0x00000000,  # is it even needed?
-            # HLSQ_DBG_ECO_CNTL = 0x0,
-            RB_UNKNOWN_8E01 = 0x00000000,
-            VPC_DBG_ECO_CNTL = 0x02000000,
-            UCHE_UNKNOWN_0E12 = 0x00000000,
-
-            RB_UNKNOWN_8E06 = 0x02080000,
-        ),
+        magic_regs = a740_magic_regs,
         raw_magic_regs = [
             [A6XXRegs.REG_A6XX_UCHE_CACHE_WAYS, 0x00040004],
             [A6XXRegs.REG_A6XX_TPL1_DBG_ECO_CNTL1, 0x00000700],
@@ -1130,25 +1239,18 @@ add_gpus([
 
             [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_80A7, 0x00000000],
 
-            [A6XXRegs.REG_A7XX_HLSQ_UNKNOWN_A9AC, 0x00000000],
             [A6XXRegs.REG_A7XX_RB_UNKNOWN_8E79,   0x00000000],
             [A6XXRegs.REG_A7XX_RB_UNKNOWN_8899,   0x00000000],
             [A6XXRegs.REG_A7XX_RB_UNKNOWN_88F5,   0x00000000],
-
-            # Shading rate group
-            [A6XXRegs.REG_A6XX_RB_UNKNOWN_88F4,   0x00000000],
-            [A6XXRegs.REG_A7XX_HLSQ_UNKNOWN_A9AD, 0x00000000],
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_80F4, 0x00000000],
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_80F5, 0x00000000],
         ],
     ))
 
 add_gpus([
-        GPUId(chip_id=0x43051401, name="FD750"), # KGSL, no speedbin data
-        GPUId(chip_id=0xffff43051401, name="FD750"), # Default no-speedbin fallback
+        GPUId(chip_id=0x43050b00, name="FD740v3"), # Quest 3
+        GPUId(chip_id=0xffff43050b00, name="FD740v3"),
     ], A6xxGPUInfo(
         CHIP.A7XX,
-        [a7xx_base, a7xx_750],
+        [a7xx_base, a7xx_gen2, A7XXProps(enable_tp_ubwc_flag_hint = True)],
         num_ccu = 6,
         tile_align_w = 96,
         tile_align_h = 32,
@@ -1156,6 +1258,42 @@ add_gpus([
         cs_shared_mem_size = 32 * 1024,
         wave_granularity = 2,
         fibers_per_sp = 128 * 2 * 16,
+        magic_regs = dict(
+            # PC_POWER_CNTL = 7,
+            TPL1_DBG_ECO_CNTL = 0x11100000,
+            GRAS_DBG_ECO_CNTL = 0x00004800,
+            SP_CHICKEN_BITS = 0x10001400,
+            UCHE_CLIENT_PF = 0x00000084,
+            # Blob uses 0x1f or 0x1f1f, however these values cause vertices
+            # corruption in some tests.
+            PC_MODE_CNTL = 0x0000003f,
+            SP_DBG_ECO_CNTL = 0x10000000,
+            RB_DBG_ECO_CNTL = 0x00000001,
+            RB_DBG_ECO_CNTL_blit = 0x00000000,  # is it even needed?
+            # HLSQ_DBG_ECO_CNTL = 0x0,
+            RB_UNKNOWN_8E01 = 0x0,
+            VPC_DBG_ECO_CNTL = 0x02000000,
+            UCHE_UNKNOWN_0E12 = 0x00000000,
+
+            RB_UNKNOWN_8E06 = 0x02080000,
+        ),
+        raw_magic_regs = a740_raw_magic_regs,
+    ))
+
+add_gpus([
+        GPUId(chip_id=0x43051401, name="FD750"), # KGSL, no speedbin data
+        GPUId(chip_id=0xffff43051401, name="FD750"), # Default no-speedbin fallback
+    ], A6xxGPUInfo(
+        CHIP.A7XX,
+        [a7xx_base, a7xx_gen3],
+        num_ccu = 6,
+        tile_align_w = 96,
+        tile_align_h = 32,
+        num_vsc_pipes = 32,
+        cs_shared_mem_size = 32 * 1024,
+        wave_granularity = 2,
+        fibers_per_sp = 128 * 2 * 16,
+        highest_bank_bit = 16,
         magic_regs = dict(
             TPL1_DBG_ECO_CNTL = 0x11100000,
             GRAS_DBG_ECO_CNTL = 0x00004800,
@@ -1205,16 +1343,11 @@ add_gpus([
 
             [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_80A7, 0x00000000],
 
-            [A6XXRegs.REG_A7XX_HLSQ_UNKNOWN_A9AC, 0x00000000],
             [A6XXRegs.REG_A7XX_RB_UNKNOWN_8899,   0x00000000],
             [A6XXRegs.REG_A7XX_RB_UNKNOWN_88F5,   0x00000000],
             [A6XXRegs.REG_A7XX_RB_UNKNOWN_8C34,   0x00000000],
 
-            # Shading rate group
-            [A6XXRegs.REG_A6XX_RB_UNKNOWN_88F4,   0x00000000],
-            [A6XXRegs.REG_A7XX_HLSQ_UNKNOWN_A9AD, 0x00000000],
             [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8008, 0x00000000],
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_80F4, 0x00000000],
 
             [0x930a, 0],
             [0x960a, 1],
@@ -1224,26 +1357,9 @@ add_gpus([
     ))
 
 template = """\
-/* Copyright (C) 2021 Google, Inc.
+/* Copyright © 2021 Google, Inc.
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
- * IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  */
 
 #include "freedreno_dev_info.h"
